local Critline = Critline
local L = LibStub("AceLocale-3.0"):GetLocale("Critline")

local checks = {
	GUILD = IsInGuild,
	PARTY = IsInGroup,
	RAID = IsInRaid,
}

local addon = Critline:AddCategory("AutoAnnounce", true)
addon:Hide()

do
	local channels = {
		"SAY",
		"GUILD",
		"PARTY",
		"RAID",
		"WHISPER",
		"CHANNEL",
	}

	for i, v in ipairs(channels) do
		channels[i] = {
			text = _G[v],
			value = v,
		}
	end
	
	local options = {
		{
			type = "CheckButton",
			label = L["Enabled"],
			setting = "enabled",
			func = function(self, checked)
				if checked then
					Critline.RegisterCallback(addon, "NewRecord")
				else
					Critline.UnregisterCallback(addon, "NewRecord")
				end
			end,
		},
		{
			type = "DropDownMenu",
			setting = "channel",
			width = 120,
			func = function(self, value)
				local target = addon.target
				if value == "WHISPER" or value == "CHANNEL" then
					target:Show()
					target:SetFocus()
				else
					target:Hide()
				end
			end,
			menu = channels,
		},
	}
	
	addon:CreateOptions(options)
	
	local target = addon:CreateEditBox(addon)
	target:SetWidth(134)
	target:SetPoint("TOP", addon.registry.channel, "BOTTOM", 0, -8)
	target:SetScript("OnEnterPressed", function(self)
		local text = self:GetText():trim()
		local channel = addon.db.profile.channel
		if channel == "WHISPER" then
			if text == "" then
				Critline:Message(SPELL_FAILED_BAD_TARGETS)
				return
			end
		elseif channel == "CHANNEL" then
			text = GetChannelName(text)
			if text == 0 then
				Critline:Message(L["Invalid channel. Please enter a valid channel name or ID."])
				return
			end
		end
		addon.db.profile.target = text
		self:ClearFocus()
	end)
	target:SetScript("OnEscapePressed", target.ClearFocus)
	target:Hide()
	addon.target = target
end

function addon:LoadSettings()
	self:LoadOptions()
	self.target:SetText(self.db.profile.target)
end

function addon:NewRecord(event, tree, spellID, spellName, periodic, amount, crit)
	local channel = self.db.profile.channel
	if checks[channel] and not checks[channel]() then
		return
	end
	SendChatMessage(format(L["New %s%s record - %s"], crit and L["critical "] or "", spellName, amount), channel, nil, self.db.profile.target)
end

local defaults = {
	profile = {
		enabled = true,
		channel = "SAY",
		target = "",
	}
}

addon.db = Critline.db:RegisterNamespace("AutoAnnounce", defaults)
addon:LoadSettings()
Critline.RegisterCallback(addon, "SettingsLoaded", "LoadSettings")